/*
 * Copyright 2018  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef __MULTI_ASSISTANT_CONFIG_H__
#define __MULTI_ASSISTANT_CONFIG_H__

#include <tizen.h>
#include <tzplatform_config.h>

#ifdef __cplusplus
extern "C"
{
#endif

#define MAX_WAKEUP_LIST_NUM 32
#define MAX_SUPPORTED_LANGUAGE_NUM 128

/**************************************************************************************
 *** Definitions for xml file
 *************************************************************************************/
#define MA_TAG_ASSISTANT_BASE					"multi-assistant"
#define MA_TAG_ASSISTANT_NAME					"name"
#define MA_TAG_ASSISTANT_APPID					"appid"
#define MA_TAG_ASSISTANT_ICON_PATH				"icon-path"
#define MA_TAG_ASSISTANT_LANGUAGE_SET			"languages"
#define MA_TAG_ASSISTANT_LANGUAGE				"language"
#define MA_TAG_ASSISTANT_WAKEUP_WORD_SET		"wakeup-words"
#define MA_TAG_ASSISTANT_WAKEUP_WORD			"wakeup-word"
#define MA_TAG_ASSISTANT_WAKEUP_ENGINE_APPID	"wakeup-engine-appid"
#define MA_TAG_ASSISTANT_CUSTOM_UI				"custom-ui"

/**************************************************************************************
 *** Definitions for ETC
 *************************************************************************************/
#define MA_RETRY_COUNT	5

#define MA_ASSISTANT_INFO		tzplatform_mkpath(TZ_USER_HOME, "share/.multiassistant/ma/1.0/assistant-info")

typedef struct {
	const char* app_id;
	const char* name;
	const char* icon_path;
	const char* wakeup_list[MAX_WAKEUP_LIST_NUM];
	const char* wakeup_language[MAX_WAKEUP_LIST_NUM];
	int cnt_wakeup;
	const char* supported_lang[MAX_SUPPORTED_LANGUAGE_NUM];
	int cnt_lang;
	const char* wakeup_engine;
	bool custom_ui_option;
} ma_assistant_info_s;

typedef int (*mas_config_assistant_info_cb)(const char* appid, const char* name, const char* icon_path,
	const char* wakeup_list[], const char* wakeup_language[], int cnt_wakeup,
	const char* supported_lang[], int cnt_lang,
	const char* wakeup_engine, bool custom_ui_option, void* user_data);
int mas_config_get_assistant_info(mas_config_assistant_info_cb callback, void* user_data);

#ifdef __cplusplus
}
#endif

/**
 * @}
 */

#endif /* __MULTI_ASSISTANT_CONFIG_H__ */
