#ifndef __MULTI_ASSISTANT_SERVICE_H__
#define __MULTI_ASSISTANT_SERVICE_H__

#include <app.h>
#include <glib.h>
#include "multi_wakeup_recognizer.h"

#ifdef  LOG_TAG
#undef  LOG_TAG
#endif
#define LOG_TAG "multi-assistant"

#define MAS_SECURE_LOG_(id, prio, tag, fmt, arg...) \
		({ do { \
				__dlog_print(id, prio, tag, "%s: %s(%d) > [SECURE_LOG] " fmt, __MODULE__, __func__, __LINE__, ##arg); \
		} while (0); })

#define MAS_LOG_(prio, tag, fmt, arg...) \
		({ do { \
				dlog_print(prio, tag, "%s: %s(%d) > " fmt, __MODULE__, __func__, __LINE__, ##arg); \
		} while (0); })

#define MAS_LOGD(fmt, args...) MAS_LOG_(DLOG_DEBUG, LOG_TAG, fmt, ##args)
#define MAS_LOGI(fmt, args...) MAS_LOG_(DLOG_INFO, LOG_TAG, fmt, ##args)
#define MAS_LOGW(fmt, args...) MAS_LOG_(DLOG_WARN, LOG_TAG, fmt, ##args)
#define MAS_LOGE(fmt, args...) MAS_LOG_(DLOG_ERROR, LOG_TAG, fmt, ##args)

#define MAS_SLOGD(fmt, args...) MAS_SECURE_LOG_(DLOG_DEBUG, LOG_TAG, fmt, ##args)
#define MAS_SLOGI(fmt, args...) MAS_SECURE_LOG_(DLOG_INFO, LOG_TAG, fmt, ##args)
#define MAS_SLOGW(fmt, args...) MAS_SECURE_LOG_(DLOG_WARN, LOG_TAG, fmt, ##args)
#define MAS_SLOGE(fmt, args...) MAS_SECURE_LOG_(DLOG_ERROR, LOG_TAG, fmt, ##args)


/**************************************************************************************
 *** Definitions for DBus
 *************************************************************************************/
#define MA_CLIENT_SERVICE_NAME			"org.tizen.multiassistant.maclient"
#define MA_CLIENT_SERVICE_OBJECT_PATH	"/org/tizen/multiassistant/maclient"
#define MA_CLIENT_SERVICE_INTERFACE		"org.tizen.multiassistant.maclient"

#define MA_UI_CLIENT_SERVICE_NAME		"org.tizen.multiassistant.mauiclient"
#define MA_UI_CLIENT_SERVICE_OBJECT_PATH"/org/tizen/multiassistant/mauiclient"
#define MA_UI_CLIENT_SERVICE_INTERFACE	"org.tizen.multiassistant.mauiclient"

#define MA_SERVER_SERVICE_NAME			"org.tizen.multiassistant.maserver"
#define MA_SERVER_SERVICE_OBJECT_PATH	"/org/tizen/multiassistant/maserver"
#define MA_SERVER_SERVICE_INTERFACE		"org.tizen.multiassistant.maserver"

/**************************************************************************************
 *** Definitions for DBus methods
 *************************************************************************************/
#define MA_METHOD_HELLO								"ma_method_hello"

#define MA_METHOD_INITIALIZE						"ma_method_initialize"
#define MA_METHOD_DEINITIALIZE						"ma_method_deinitialize"
#define MA_METHOD_GET_RECORDING_AUDIO_FORMAT		"ma_method_get_recording_audio_format"
#define MA_METHOD_GET_RECORDING_AUDIO_SOURCE_TYPE	"ma_method_get_recording_audio_source_type"
#define MA_METHOD_SEND_ASR_RESULT					"ma_method_send_asr_result"
#define MA_METHOD_SEND_RESULT						"ma_method_send_result"
#define MA_METHOD_SEND_RECOGNITION_RESULT			"ma_method_send_recognition_result"
#define MA_METHOD_START_STREAMING_AUDIO_DATA		"ma_method_start_streaming_audio_data"
#define MA_METHOD_STOP_STREAMING_AUDIO_DATA			"ma_method_stop_streaming_audio_data"
#define MA_METHOD_UPDATE_VOICE_FEEDBACK_STATE		"ma_method_update_voice_feedback_state"
#define MA_METHOD_SEND_ASSISTANT_SPECIFIC_COMMAND	"ma_method_send_assistant_specific_command"
#define MA_METHOD_SET_BACKGROUND_VOLUME				"ma_method_set_background_volume"
#define MA_METHOD_SET_PREPROCESSING_ALLOW_MODE		"ma_method_set_preprocessing_allow_mode"
#define MA_METHOD_SEND_PREPROCESSING_RESULT			"ma_method_send_preprocessing_result"
#define MA_METHOD_SET_WAKE_WORD_AUDIO_REQUIRE_FLAG	"ma_method_set_wake_word_audio_require_flag"
#define MA_METHOD_SET_ASSISTANT_LANGUAGE			"ma_method_set_assistant_language"
#define MA_METHOD_ERROR								"ma_method_error"

#define MA_UI_METHOD_INITIALIZE						"ma_ui_method_initialize"
#define MA_UI_METHOD_DEINITIALIZE					"ma_ui_method_deinitialize"
#define MA_UI_METHOD_CHANGE_ASSISTANT				"ma_ui_method_change_assistant"

#define MAS_METHOD_HELLO							"mas_method_hello"
#define MAS_METHOD_ACTIVE_STATE_CHANGE				"mas_method_active_state_change"
#define MAS_METHOD_STREAMING_AUDIO_DATA				"mas_method_streaming_audio_data"
#define MAS_METHOD_WAKEUP_ENGINE_COMMAND			"mas_method_wakeup_engine_command"
#define MAS_METHOD_ERROR							"mas_method_error"
#define MAS_METHOD_SEND_PREPROCESSING_INFORMATION	"mas_method_send_preprocessing_information"
#define MAS_METHOD_AUDIO_STREAMING_DATA_SECTION		"mas_method_audio_streaming_data_section"
#define MAS_METHOD_SEND_PREPROCESSING_RESULT		"mas_method_send_preprocessing_result"

#define MAS_UI_METHOD_SEND_ASR_RESULT				"mas_ui_method_send_asr_result"
#define MAS_UI_METHOD_SEND_RESULT					"mas_ui_method_send_result"
#define MAS_UI_METHOD_CHANGE_ASSISTANT				"mas_ui_method_change_assistant"
#define MAS_UI_METHOD_ERROR							"mas_ui_method_error"
#define MAS_UI_METHOD_SEND_RECOGNITION_RESULT		"mas_ui_method_send_recognition_result"
#define MAS_UI_METHOD_ENABLE_COMMON_UI				"mas_ui_method_enable_common_ui"

#endif /* __MULTI_ASSISTANT_SERVICE_H__ */
