/*
 * Copyright 2018 Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _MULTI_ASSISTANT_SERVICE_H_
#define _MULTI_ASSISTANT_SERVICE_H_

#include <dlog/dlog.h>


#ifdef __cplusplus
extern "C" {
#endif

#define MAX_APPID_LEN 255

int mas_client_initialize(int pid);

int mas_client_deinitialize(int pid);

int mas_client_get_audio_format(int pid, int* rate, int* channel, int* audio_type);

int mas_client_get_audio_source_type(int pid, char** type);

int mas_client_request_speech_data(int pid);

int mas_client_send_asr_result(int pid, int event, char* asr_result);

int mas_client_send_result(int pid, char* display_text, char* utterance_text, char* result_json);

int mas_client_send_recognition_result(int pid, int result);

int mas_client_start_streaming_audio_data(int pid, int type);

int mas_client_stop_streaming_audio_data(int pid, int type);

int mas_client_update_voice_feedback_state(int pid, int state);

int mas_client_send_assistant_specific_command(int pid, const char *command);

int mas_client_set_background_volume(int pid, double ratio);

int mas_client_set_preprocessing_allow_mode(int pid, int mode, const char* appid);

int mas_client_send_preprocessing_result(int pid, bool result);

int mas_ui_client_initialize(int pid);

int mas_ui_client_deinitialize(int pid);

int mas_ui_client_change_assistant(const char* appid);

int mas_get_current_client_pid();

int mas_get_current_preprocessing_client_pid();

int mas_get_client_pid_by_wakeup_word(const char *wakeup_word);

int mas_get_client_pid_by_appid(const char *appid);

bool mas_get_client_custom_ui_option_by_appid(const char *appid);

const char* mas_get_client_appid_by_wakeup_word(const char *wakeup_word);

int mas_set_current_client_by_wakeup_word(const char *wakeup_word);

int mas_set_current_client_by_appid(const char *appid);

int mas_launch_client_by_wakeup_word(const char *wakeup_word);

int mas_prelaunch_default_assistant();

typedef enum {
	CLIENT_LAUNCH_MODE_ACTIVATION,
	CLIENT_LAUNCH_MODE_PRELAUNCH,
} CLIENT_LAUNCH_MODE;
int mas_launch_client_by_appid(const char *appid, CLIENT_LAUNCH_MODE launch_mode);

int mas_process_voice_key_event(bool pressed);

typedef enum {
	PREPROCESSING_STATE_NONE,
	PREPROCESSING_STATE_WAKEUP_PREPROCESS_ENABLED,
	PREPROCESSING_STATE_WAKEUP_PREPROCESS_DISABLED,
	PREPROCESSING_STATE_PREPROCESSING_UTTERANCE,
	PREPROCESSING_STATE_PREPROCESSING_FOLLOW_UP,
} PREPROCESSING_STATE;

typedef enum {
	PREPROCESSING_STATE_EVENT_ASSISTANT_ACTIVATED,
	PREPROCESSING_STATE_EVENT_PREPROCESSING_ALLOW_MODE_CHANGED,
	PREPROCESSING_STATE_EVENT_UTTERANCE_STREAMING_STARTED,
	PREPROCESSING_STATE_EVENT_FOLLOW_UP_STREAMING_STARTED,
	PREPROCESSING_STATE_EVENT_PREPROCESSING_SUCCEEDED,
	PREPROCESSING_STATE_EVENT_PREPROCESSING_FAILED,
} PREPROCESSING_STATE_EVENT;

int mas_process_preprocessing_state_event(PREPROCESSING_STATE_EVENT event);


#ifdef __cplusplus
}
#endif

#endif /* _MULTI_ASSISTANT_SERVICE_H_ */
