/*
 * Copyright 2018  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _MULTI_ASSISTANT_SERVICE_PLUGIN_H_
#define _MULTI_ASSISTANT_SERVICE_PLUGIN_H_

#include <dlog/dlog.h>

#include "multi_wakeup_recognizer.h"
#include "multi_assistant_main.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
	MA_PLUGIN_EVENT_VOICE_KEY_PRESSED = 0,
	MA_PLUGIN_EVENT_VOICE_KEY_RELEASED,
	MA_PLUGIN_EVENT_VOICE_KEY_RELEASED_AFTER_PUSH = MA_PLUGIN_EVENT_VOICE_KEY_RELEASED,
	MA_PLUGIN_EVENT_VOICE_KEY_RELEASED_AFTER_TAP,
} ma_plugin_event_e;

typedef struct {
	int plugin_version;
	bool ui_panel_enabled;
} ma_plugin_settings;

int multi_assistant_service_plugin_initialize(void);

int multi_assistant_service_plugin_deinitialize(void);

int multi_assistant_service_plugin_get_settings(ma_plugin_settings **settings, size_t *struct_size);

int multi_assistant_service_plugin_set_language(const char* language);

int multi_assistant_service_plugin_add_assistant_wakeup_word(const char* appid, const char* wakeup_word, const char* language);

int multi_assistant_service_plugin_add_assistant_language(const char* appid, const char* language);

int multi_assistant_service_plugin_set_assistant_wakeup_engine(const char* appid, const char* engine);

int multi_assistant_service_plugin_set_default_assistant(const char* appid);

int multi_assistant_service_plugin_get_default_assistant(const char** appid);

int multi_assistant_service_plugin_activate(void);

int multi_assistant_service_plugin_deactivate(void);

int multi_assistant_service_plugin_update_voice_feedback_state(const char* appid, int state);

int multi_assistant_service_plugin_send_assistant_specific_command(const char *appid, const char* command);

int multi_assistant_service_plugin_set_background_volume(const char *appid, double ratio);

int multi_assistant_service_plugin_update_recognition_result(const char* appid, int result);

int multi_assistant_service_plugin_set_preprocessing_allow_mode(const char* appid, int mode, const char* preprocessing_appid);

int multi_assistant_service_plugin_process_event(int event, void* data, int len);

int multi_assistant_service_plugin_start_streaming_utterance_data(void);

int multi_assistant_service_plugin_stop_streaming_utterance_data(void);

int multi_assistant_service_plugin_start_streaming_previous_utterance_data(void);

int multi_assistant_service_plugin_stop_streaming_previous_utterance_data(void);

int multi_assistant_service_plugin_start_streaming_follow_up_data(void);

int multi_assistant_service_plugin_stop_streaming_follow_up_data(void);

int multi_assistant_service_plugin_get_recording_audio_format(int* rate, int* channel, int* audio_type);

int multi_assistant_service_plugin_get_recording_audio_source_type(char **type);

int multi_assistant_service_plugin_set_callbacks(void);

int multi_assistant_service_plugin_set_wakeup_event_callback(wakeup_service_wakeup_event_cb callback, void* user_data);

int multi_assistant_service_plugin_set_utterance_streaming_callback(wakeup_service_speech_streaming_cb callback, void* user_data);

int multi_assistant_service_plugin_set_previous_utterance_streaming_callback(wakeup_service_speech_streaming_cb callback, void* user_data);

int multi_assistant_service_plugin_set_follow_up_streaming_callback(wakeup_service_speech_streaming_cb callback, void* user_data);

int multi_assistant_service_plugin_set_speech_status_callback(wakeup_service_speech_status_cb callback, void* user_data);

int multi_assistant_service_plugin_set_error_callback(wakeup_service_error_cb callback, void* user_data);


#define MA_WAKEUP_MANAGER_PATH tzplatform_mkpath(tzplatform_getid("TZ_SYS_RO_SHARE"), "multiassistant/")

#define MA_DEFAULT_WAKEUP_MANAGER_FILENAME "libma-wakeup-manager.so"


#define MA_WAKEUP_MANAGER_FUNC_INITIALIZE "wakeup_manager_initialize"
typedef int (*wakeup_manager_initialize)(void);
#define MA_WAKEUP_MANAGER_FUNC_DEINITIALIZE "wakeup_manager_deinitialize"
typedef int (*wakeup_manager_deinitialize)(void);
#define MA_WAKEUP_MANAGER_FUNC_GET_SETTINGS "wakeup_manager_get_settings"
typedef int (*wakeup_manager_get_settings)(ma_plugin_settings **settings, size_t *struct_size);
#define MA_WAKEUP_MANAGER_FUNC_ADD_ASSISTANT_WAKEUP_WORD "wakeup_manager_add_assistant_wakeup_word"
typedef int (*wakeup_manager_add_assistant_wakeup_word)(const char* appid, const char* wakeup_word, const char* language);
#define MA_WAKEUP_MANAGER_FUNC_ADD_ASSISTANT_LANGUAGE "wakeup_manager_add_assistant_language"
typedef int (*wakeup_manager_add_assistant_language)(const char* appid, const char* language);
#define MA_WAKEUP_MANAGER_FUNC_SET_ASSISTANT_WAKEUP_ENGINE "wakeup_manager_set_assistant_wakeup_engine"
typedef int (*wakeup_manager_set_assistant_wakeup_engine)(const char* appid, const char* engine);
#define MA_WAKEUP_MANAGER_FUNC_SET_DEFAULT_ASSISTANT "wakeup_manager_set_default_assistant"
typedef int (*wakeup_manager_set_default_assistant)(const char* appid);
#define MA_WAKEUP_MANAGER_FUNC_GET_DEFAULT_ASSISTANT "wakeup_manager_get_default_assistant"
typedef int (*wakeup_manager_get_default_assistant)(const char** appid);
#define MA_WAKEUP_MANAGER_FUNC_SET_LANGUAGE "wakeup_manager_set_language"
typedef int (*wakeup_manager_set_language)(const char* language);
#define MA_WAKEUP_MANAGER_FUNC_ACTIVATE "wakeup_manager_activate"
typedef int (*wakeup_manager_activate)(void);
#define MA_WAKEUP_MANAGER_FUNC_DEACTIVATE "wakeup_manager_deactivate"
typedef int (*wakeup_manager_deactivate)(void);
#define MA_WAKEUP_MANAGER_FUNC_UPDATE_VOICE_FEEDBACK_STATE "wakeup_manager_update_voice_feedback_state"
typedef int (*wakeup_manager_update_voice_feedback_state)(const char* appid, int state);
#define MA_WAKEUP_MANAGER_FUNC_SEND_ASSISTANT_SPECIFIC_COMMAND "wakeup_manager_send_assistant_specific_command"
typedef int (*wakeup_manager_send_assistant_specific_command)(const char* appid, const char* command);
#define MA_WAKEUP_MANAGER_FUNC_SET_BACKGROUND_VOLUME "wakeup_manager_set_background_volume"
typedef int (*wakeup_manager_set_background_volume)(const char* appid, double ratio);
#define MA_WAKEUP_MANAGER_FUNC_UPDATE_RECOGNITION_RESULT "wakeup_manager_update_recognition_result"
typedef int (*wakeup_manager_update_recognition_result)(const char* appid, int result);
#define MA_WAKEUP_MANAGER_FUNC_PROCESS_EVENT "wakeup_manager_process_event"
typedef int (*wakeup_manager_process_event)(int event, void* data, int len);
#define MA_WAKEUP_MANAGER_FUNC_START_STREAMING_UTTERANCE_DATA "wakeup_manager_start_streaming_utterance_data"
typedef int (*wakeup_manager_start_streaming_utterance_data)(void);
#define MA_WAKEUP_MANAGER_FUNC_STOP_STREAMING_UTTERANCE_DATA "wakeup_manager_stop_streaming_utterance_data"
typedef int (*wakeup_manager_stop_streaming_utterance_data)(void);
#define MA_WAKEUP_MANAGER_FUNC_START_STREAMING_PREVIOUS_UTTERANCE_DATA "wakeup_manager_start_streaming_previous_utterance_data"
typedef int (*wakeup_manager_start_streaming_previous_utterance_data)(void);
#define MA_WAKEUP_MANAGER_FUNC_STOP_STREAMING_PREVIOUS_UTTERANCE_DATA "wakeup_manager_stop_streaming_previous_utterance_data"
typedef int (*wakeup_manager_stop_streaming_previous_utterance_data)(void);
#define MA_WAKEUP_MANAGER_FUNC_START_STREAMING_FOLLOW_UP_DATA "wakeup_manager_start_streaming_follow_up_data"
typedef int (*wakeup_manager_start_streaming_follow_up_data)(void);
#define MA_WAKEUP_MANAGER_FUNC_STOP_STREAMING_FOLLOW_UP_DATA "wakeup_manager_stop_streaming_follow_up_data"
typedef int (*wakeup_manager_stop_streaming_follow_up_data)(void);
#define MA_WAKEUP_MANAGER_FUNC_GET_AUDIO_FORMAT "wakeup_manager_get_audio_format"
typedef int (*wakeup_manager_get_audio_format)(int* rate, int* channel, int* audio_type);
#define MA_WAKEUP_MANAGER_FUNC_GET_AUDIO_SOURCE_TYPE "wakeup_manager_get_audio_source_type"
typedef int (*wakeup_manager_get_audio_source_type)(char** type);
#define MA_WAKEUP_MANAGER_FUNC_SET_WAKE_WORD_AUDIO_REQUIRE_FLAG "wakeup_manager_set_wake_word_audio_require_flag"
typedef int (*wakeup_manager_set_wake_word_audio_require_flag)(bool require);
#define MA_WAKEUP_MANAGER_FUNC_SET_ASSISTANT_LANGUAGE "wakeup_manager_set_assistant_language"
typedef int (*wakeup_manager_set_assistant_language)(const char* appid, const char* language);
#define MA_WAKEUP_MANAGER_FUNC_SET_WAKEUP_EVENT_CALLBACK "wakeup_manager_set_wakeup_event_callback"
typedef int (*wakeup_manager_set_wakeup_event_callback)(wakeup_service_wakeup_event_cb callback, void* user_data);
#define MA_WAKEUP_MANAGER_FUNC_SET_UTTERANCE_STREAMING_CALLBACK "wakeup_manager_set_utterance_streaming_callback"
typedef int (*wakeup_manager_set_utterance_streaming_callback)(wakeup_service_speech_streaming_cb callback, void* user_data);
#define MA_WAKEUP_MANAGER_FUNC_SET_PREVIOUS_UTTERANCE_STREAMING_CALLBACK "wakeup_manager_set_previous_utterance_streaming_callback"
typedef int (*wakeup_manager_set_previous_utterance_streaming_callback)(wakeup_service_speech_streaming_cb callback, void* user_data);
#define MA_WAKEUP_MANAGER_FUNC_SET_FOLLOW_UP_STREAMING_CALLBACK "wakeup_manager_set_follow_up_streaming_callback"
typedef int (*wakeup_manager_set_follow_up_streaming_callback)(wakeup_service_speech_streaming_cb callback, void* user_data);
#define MA_WAKEUP_MANAGER_FUNC_SET_SPEECH_STATUS_CALLBACK "wakeup_manager_set_speech_status_callback"
typedef int (*wakeup_manager_set_speech_status_callback)(wakeup_service_speech_status_cb callback, void* user_data);
#define MA_WAKEUP_MANAGER_FUNC_SET_SETTING_CHANGED_CALLBACK "wakeup_manager_set_setting_changed_callback"
typedef int (*wakeup_manager_set_setting_changed_callback)(wakeup_service_setting_changed_cb callback, void* user_data);
#define MA_WAKEUP_MANAGER_FUNC_SET_ERROR_CALLBACK "wakeup_manager_set_error_callback"
typedef int (*wakeup_manager_set_error_callback)(wakeup_service_error_cb callback, void* user_data);
#define MA_WAKEUP_MANAGER_FUNC_SET_STREAMING_SECTION_CHANGED_CALLBACK "wakeup_manager_set_streaming_section_changed_callback"
typedef int (*wakeup_manager_set_streaming_section_changed_callback)(wakeup_service_streaming_section_changed_cb callback, void* user_data);

typedef struct {
	wakeup_manager_initialize								 initialize;
	wakeup_manager_deinitialize								 deinitialize;
	wakeup_manager_get_settings								 get_settings;
	wakeup_manager_add_assistant_wakeup_word				 add_assistant_wakeup_word;
	wakeup_manager_add_assistant_language					 add_assistant_language;
	wakeup_manager_set_assistant_wakeup_engine				 set_assistant_wakeup_engine;
	wakeup_manager_set_default_assistant					 set_default_assistant;
	wakeup_manager_get_default_assistant					 get_default_assistant;
	wakeup_manager_set_language								 set_language;
	wakeup_manager_activate									 activate;
	wakeup_manager_deactivate								 deactivate;
	wakeup_manager_update_voice_feedback_state				 update_voice_feedback_state;
	wakeup_manager_send_assistant_specific_command			 send_assistant_specific_command;
	wakeup_manager_set_background_volume					 set_background_volume;
	wakeup_manager_update_recognition_result				 update_recognition_result;
	wakeup_manager_process_event							 process_event;
	wakeup_manager_start_streaming_utterance_data			 start_streaming_utterance_data;
	wakeup_manager_stop_streaming_utterance_data			 stop_streaming_utterance_data;
	wakeup_manager_start_streaming_previous_utterance_data	 start_streaming_previous_utterance_data;
	wakeup_manager_stop_streaming_previous_utterance_data	 stop_streaming_previous_utterance_data;
	wakeup_manager_start_streaming_follow_up_data			 start_streaming_follow_up_data;
	wakeup_manager_stop_streaming_follow_up_data			 stop_streaming_follow_up_data;
	wakeup_manager_get_audio_format							 get_audio_format;
	wakeup_manager_get_audio_source_type					 get_audio_source_type;
	wakeup_manager_set_wake_word_audio_require_flag			 set_wake_word_audio_require_flag;
	wakeup_manager_set_assistant_language					 set_assistant_language;
	wakeup_manager_set_wakeup_event_callback				 set_wakeup_event_callback;
	wakeup_manager_set_utterance_streaming_callback			 set_utterance_streaming_callback;
	wakeup_manager_set_previous_utterance_streaming_callback	set_previous_utterance_streaming_callback;
	wakeup_manager_set_follow_up_streaming_callback			 set_follow_up_streaming_callback;
	wakeup_manager_set_speech_status_callback				 set_speech_status_callback;
	wakeup_manager_set_setting_changed_callback				 set_setting_changed_callback;
	wakeup_manager_set_error_callback						 set_error_callback;
	wakeup_manager_set_streaming_section_changed_callback	 set_streaming_section_changed_callback;
} wakeup_manager_interface;

#ifdef __cplusplus
}
#endif

#endif /* _MULTI_ASSISTANT_SERVICE_PLUGIN_H_ */
