/*
 * Copyright 2018  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _MULTI_WAKEUP_RECOGNIZER_H_
#define _MULTI_WAKEUP_RECOGNIZER_H_

#include <dlog/dlog.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
	const char *wakeup_appid;
	const char *wakeup_word;
	const char *wakeup_language;
	const char *wakeup_voice_id;
	const char *wakeup_engine;
	float wakeup_confidence_score;

	long wakeup_start_time;
	long wakeup_end_time;
	bool wakeup_time_valid;

	const void *extra_data;
	int extra_data_length;
	const char *extra_data_description;
} wakeup_event_info;

typedef enum {
	WAKEUP_SPEECH_STREAMING_EVENT_FAIL = -1,    /**< Failed */
	WAKEUP_SPEECH_STREAMING_EVENT_START = 1,    /**< Start event */
	WAKEUP_SPEECH_STREAMING_EVENT_CONTINUE = 2, /**< Continue event */
	WAKEUP_SPEECH_STREAMING_EVENT_FINISH = 3    /**< Finish event */
} wakeup_speech_streaming_event_e;

typedef enum {
	WAKEUP_SPEECH_STATUS_NONE = -1, 					/**< None */
	WAKEUP_SPEECH_STATUS_BEGINNING_POINT_DETECTED = 1,	/**< Beginning point of speech is detected */
	WAKEUP_SPEECH_STATUS_END_POINT_DETECTED = 2 		/**< End point of speech is detected */
} wakeup_speech_status_e;

typedef enum {
	WAKEUP_ASR_RESULT_EVENT_FINAL = 0, 	/**< Event when either the full matched or the final result is delivered */
	WAKEUP_ASR_RESULT_EVENT_PARTIAL, 	/**< Event when the partial matched result is delivered */
	WAKEUP_ASR_RESULT_EVENT_ERROR 		/**< Event when the recognition has failed */
} wakeup_asr_result_event_e;

typedef enum {
	MA_AUDIO_STREAMING_DATA_TYPE_CURRENT_UTTERANCE = 0,		/**< Current utterance */
	MA_AUDIO_STREAMING_DATA_TYPE_PREVIOUS_UTTERANCE,			/**< Previous utterance */
	MA_AUDIO_STREAMING_DATA_TYPE_FOLLOW_UP_SPEECH				/**< Follow-up speech */
} ma_audio_streaming_data_type_e;

typedef enum {
	MA_ACTIVE_STATE_INACTIVE = 0,	/**< 'Inactive' state */
	MA_ACTIVE_STATE_ACTIVE,	/**< 'Active' state */
	MA_ACTIVE_STATE_PREPROCESSING,	/**< 'Preprocessing' state */
} ma_active_state_e;

typedef enum {
	MA_RECOGNITION_RESULT_EVENT_SUCCESS = 0,		/**< Recognition succeeded */
	MA_RECOGNITION_RESULT_EVENT_EMPTY_TEXT,			/**< No text result recognized */
	MA_RECOGNITION_RESULT_EVENT_ERROR,				/**< Unknown error occurred */
	MA_RECOGNITION_RESULT_EVENT_FALSE_TRIGGER		/**< Turned out to be a false trigger */
} ma_recognition_result_event_e;

typedef enum {
	MA_PREPROCESSING_ALLOW_NONE = 0,	/**< No preprocessing allowed */
	MA_PREPROCESSING_ALLOW_UTTERANCE,	/**< Preprocessing allowed for utterance audio only */
	MA_PREPROCESSING_ALLOW_FOLLOW_UP,	/**< Preprocessing allowed for follow-up audio only */
	MA_PREPROCESSING_ALLOW_ALL,			/**< Preprocessing allowed for all audio */
} ma_preprocessing_allow_mode_e;

typedef enum {
	MA_AUDIO_STREAMING_DATA_SECTION_UTTERANCE = 0,	/**< Utterance section started */
	MA_AUDIO_STREAMING_DATA_SECTION_WAKE_WORD,		/**< Wake word section started */
} ma_audio_streaming_data_section_e;

typedef void (*ma_audio_streaming_data_section_changed_cb)(ma_audio_streaming_data_section_e section, void* user_data);

typedef void (*wakeup_service_wakeup_event_cb)(wakeup_event_info wakeup_info, const char* wakeup_word, void* user_data);

typedef void (*wakeup_service_speech_streaming_cb)(wakeup_speech_streaming_event_e event, unsigned char* buffer, int len, void *user_data);

typedef void (*wakeup_service_speech_status_cb)(wakeup_speech_status_e status, void *user_data);

typedef void (*wakeup_service_setting_changed_cb)(void *user_data);

typedef void (*wakeup_service_error_cb)(int error, const char* err_msg, void* user_data);

typedef void (*wakeup_service_streaming_section_changed_cb)(ma_audio_streaming_data_section_e section, void* user_data);

#ifdef __cplusplus
}
#endif

#endif /* _MULTI_WAKEUP_RECOGNIZER_H_ */
