/*
 * Copyright 2018  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _DEPENDENCY_DEFAULT_H_
#define _DEPENDENCY_DEFAULT_H_

#include <tizen.h>

#ifndef LOG_TAG
#define LOG_TAG "dependency_default"
#endif

#ifdef __cplusplus
extern "C" {
#endif

typedef enum {
	WAKEUP_SPEECH_STREAMING_EVENT_FAIL = -1,    /**< Failed */
	WAKEUP_SPEECH_STREAMING_EVENT_START = 1,    /**< Start event */
	WAKEUP_SPEECH_STREAMING_EVENT_CONTINUE = 2, /**< Continue event */
	WAKEUP_SPEECH_STREAMING_EVENT_FINISH = 3    /**< Finish event */
} wakeup_speech_streaming_event_e;

typedef enum {
	MA_PLUGIN_EVENT_VOICE_KEY_PRESSED = 0,
	MA_PLUGIN_EVENT_VOICE_KEY_RELEASED,
	MA_PLUGIN_EVENT_VOICE_KEY_RELEASED_AFTER_PUSH = MA_PLUGIN_EVENT_VOICE_KEY_RELEASED,
	MA_PLUGIN_EVENT_VOICE_KEY_RELEASED_AFTER_TAP,
} ma_plugin_event_e;

typedef enum
{
	RECORDING_SESSION_WAKE_WORD,
	RECORDING_SESSION_UTTERANCE,
	RECORDING_SESSION_FOLLOW_UP,
} recording_session;

typedef void (*mas_dependency_error_cb)(int error, const char* err_msg, void* user_data);

typedef int (*mas_proxy_process_event)(int event, void* data, int len);
typedef int (*mas_proxy_feed_audio_data)(wakeup_speech_streaming_event_e event, void* buffer, int len);

typedef struct {
	mas_proxy_process_event						process_event;
	mas_proxy_feed_audio_data					feed_audio_data;
} mas_proxy_interface;

EXPORT_API int mas_dependency_initialize(mas_proxy_interface interfaces, int *dependency_version);
EXPORT_API int mas_dependency_deinitialize(void);
EXPORT_API int mas_dependency_set_error_callback(mas_dependency_error_cb callback, void* user_data);
EXPORT_API int mas_dependency_start_recording(void);
EXPORT_API int mas_dependency_stop_recording(void);
EXPORT_API int mas_dependency_set_recording_session(unsigned int session);
EXPORT_API int mas_dependency_set_background_volume(double ratio);
EXPORT_API int mas_dependency_get_audio_format(int* rate, int* channel, int* audio_type);
EXPORT_API int mas_dependency_get_audio_source_type(char** type);

#ifdef __cplusplus
}
#endif

#endif /* _DEPENDENCY_DEFAULT_H_ */
