#include "dependency_default.h"
#include "dependency_default_audio.h"
#include "dependency_default_button.h"

#include <dlog.h>
#include <service_app_extension.h>

#include <thread>

static mas_proxy_interface g_proxy_interface;
const int g_dependency_version = 1;

static bool g_should_exit = false;

int mas_dependency_initialize(mas_proxy_interface interface, int *dependency_version)
{
	g_proxy_interface = interface;

	dependency_default_audio_initialize(interface);
	dependency_default_button_initialize(interface);

	if (dependency_version) {
		*dependency_version = g_dependency_version;
	}

	/* If the service process has to terminate in certain circumstances,
		set g_should_exit variable to true as below. */
	// g_should_exit = true;

	if (g_should_exit) {
		LOGE("g_should_exit value contains positive value, registering timer");
		std::thread([]() {
			std::this_thread::sleep_for(std::chrono::milliseconds(1000));
			LOGE("Now trying to exit");
			mas_dependency_deinitialize();
			service_app_exit_without_restart();
		}).detach();
	}

	return 0;
}

int mas_dependency_deinitialize(void)
{
	dependency_default_audio_deinitialize();
	dependency_default_button_deinitialize();

	return 0;
}

int mas_dependency_set_error_callback(mas_dependency_error_cb callback, void* user_data)
{
	return 0;
}

int mas_dependency_start_recording(void)
{
	dependency_default_audio_start_recording();
	return 0;
}

int mas_dependency_stop_recording(void)
{
	dependency_default_audio_stop_recording();
	return 0;
}

int mas_dependency_set_recording_session(unsigned int session)
{
	dependency_default_audio_set_recording_session(session);
	return 0;
}

int mas_dependency_set_background_volume(double ratio)
{
	dependency_default_audio_set_background_volume(ratio);
	return 0;
}

int mas_dependency_get_audio_format(int* rate, int* channel, int* audio_type)
{
	dependency_default_audio_get_audio_format(rate, channel, audio_type);
	return 0;
}

int mas_dependency_get_audio_source_type(char** type)
{
	dependency_default_audio_get_audio_source_type(type);
	return 0;
}
