#include "dependency_tv.h"
#include "dependency_tv_audio.h"

static bool g_voice_key_pressed = false;
static mas_proxy_interface g_proxy_interface;

#include <stdlib.h>
#include <thread>
#include <atomic>
#include <chrono>

#include <dlog.h>
#include <sound_manager.h>
#include <sound_manager_internal.h>

using namespace std;

#include <bluetooth_product.h>
#include <farfield-voice-api.h>
#include <vconf.h>

#define SMART_CONTROL_EXTEND_CMD	0x03
#define SMART_CONTROL_START_CMD		0x04

#define VC_AUDIO_ID_BLUETOOTH "VC_AUDIO_ID_BLUETOOTH"
#define VC_AUDIO_ID_MIC "VC_FARFIELD_VOICE_VD"

/* Sound buf save for test */
#if 0
#define BUF_SAVE_MODE
#endif

#ifdef BUF_SAVE_MODE
static char g_temp_file_name[128] = {'\0', };

static FILE* g_pFile = NULL;

static int g_count = 1;
#endif

static int g_bt_extend_count;
static farfield_voice_h g_farfieldvoice_h = NULL;

static sound_stream_info_h g_volume_stream = NULL;
static virtual_sound_stream_h g_virtual_sound_stream = NULL;
#define AUDIO_SOURCE_TYPE_LEN 64
static char g_audio_source_type[AUDIO_SOURCE_TYPE_LEN];

static long get_current_milliseconds_after_epoch()
{
	auto now = chrono::system_clock::now();
	auto now_ms = chrono::time_point_cast<chrono::milliseconds>(now);
	/* number of milliseconds since the epoch of system_clock */
	auto value = now_ms.time_since_epoch();

	return value.count();
}

static void set_audio_source_type(const char* src_type)
{
	if (0 != strncmp(src_type, g_audio_source_type, AUDIO_SOURCE_TYPE_LEN)) {
		snprintf(g_audio_source_type, AUDIO_SOURCE_TYPE_LEN, "%s", src_type);
		g_audio_source_type[AUDIO_SOURCE_TYPE_LEN - 1] = '\0';
		LOGE("[Recorder] source_type [%s]", g_audio_source_type);

		// TODO::
		vconf_set_str("db/farfieldvoice/devicetype", src_type);
	}
}

static void _bt_cb_hid_state_changed(int result, bool connected, const char *remote_address, void *user_data)
{
	LOGD("[Recorder] Bluetooth Event [%d] Received address [%s]", result, remote_address);
	return;
}

static void _bt_hid_audio_data_receive_cb(bt_hid_voice_data_s *voice_data, void *user_data)
{
	static int g_buffer_count = 0;
	if (nullptr == voice_data) return;

	set_audio_source_type(VC_AUDIO_ID_BLUETOOTH);

	g_proxy_interface.feed_audio_data(WAKEUP_SPEECH_STREAMING_EVENT_CONTINUE,
		voice_data->audio_buf, voice_data->length);

#ifdef BUF_SAVE_MODE
	/* write pcm buffer */
	if (g_pFile)
		fwrite(voice_data->audio_buf, 1, voice_data->length, g_pFile);
#endif

	if (0 == g_buffer_count || 0 == g_buffer_count % 50) {
		LOGD("[Recorder][%d] Recording... : read_size(%d)", g_buffer_count, voice_data->length);

		if (0 == g_bt_extend_count % 5 && 0 != g_buffer_count) {
			const unsigned char input_data[2] = {SMART_CONTROL_EXTEND_CMD, 0x10 };
			if (BT_ERROR_NONE != bt_hid_send_rc_command(NULL, input_data, sizeof(input_data))) {
				LOGE("[Recorder ERROR] Fail bt_hid_send_rc_command");
			} else {
				LOGD("[Recorder] Extend bt audio recorder");
			}
		}
		g_bt_extend_count++;

		if (100000 == g_buffer_count) {
			g_buffer_count = 0;
		}
	}

	g_buffer_count++;

	return;
}

static void stereo_to_mono_pcm(char* mono_buffer, const char* stereo_buffer, int len)
{
	int cnt = 0;
	while (cnt < len) {
		/* The code below collects audio data from only one channel for testing */
		mono_buffer[cnt / 2] = stereo_buffer[cnt];
		mono_buffer[cnt / 2 + 1] = stereo_buffer[cnt + 1];
		cnt += 4;
	}
}

static void _ffv_audio_function_cb(void* data, unsigned int length, void* user_data)
{
	/* When voice key is pressed, _bt_hid_audio should receive audio data */
	if (g_voice_key_pressed) return;

	set_audio_source_type(VC_AUDIO_ID_MIC);

	static int g_buffer_count = 0;
	if (0 == g_buffer_count || 0 == g_buffer_count % 50) {
		LOGD("[Recorder INFO] farfield audio function callback is invoked");

		if (100000 == g_buffer_count) {
			g_buffer_count = 0;
		}
	}
	g_buffer_count++;

#ifdef STEREO_TO_MONO
	int mono_length = length / 2;
	char* mono_buffer = (char*)calloc(length, sizeof(char));
	if (!mono_buffer) return;
	stereo_to_mono_pcm(mono_buffer, (const char*)data, length);

	g_proxy_interface.feed_audio_data(WAKEUP_SPEECH_STREAMING_EVENT_CONTINUE,
		mono_buffer, mono_length);

#ifdef BUF_SAVE_MODE
	/* write pcm buffer */
	if (g_pFile)
		fwrite(mono_buffer, 1, mono_length, g_pFile);
#endif

	if (mono_buffer) free(mono_buffer);
	mono_buffer = NULL;
#else
	g_proxy_interface.feed_audio_data(WAKEUP_SPEECH_STREAMING_EVENT_CONTINUE,
		data, length);
#ifdef BUF_SAVE_MODE
	/* write pcm buffer */
	if (g_pFile)
		fwrite(data, 1, length, g_pFile);
#endif
#endif
}

void dependency_tv_audio_initialize(mas_proxy_interface interfaces)
{
	g_proxy_interface = interfaces;

	bool is_bt_failed = false;

	if (false == is_bt_failed && BT_ERROR_NONE != bt_product_init()) {
		LOGE("[Recorder ERROR] Fail to init bt");
		is_bt_failed = true;
	}

	if (false == is_bt_failed && BT_ERROR_NONE != bt_hid_host_initialize(_bt_cb_hid_state_changed, NULL)) {
		LOGE("[Recorder ERROR] Fail bt_hid_host_initialize()");
		is_bt_failed = true;
	}

	if (false == is_bt_failed && BT_ERROR_NONE != bt_hid_set_audio_data_receive_cb(_bt_hid_audio_data_receive_cb, NULL)) {
		LOGE("[Recorder ERROR] Fail bt_hid_set_audio_data_receive_cb()");
		is_bt_failed = true;
	}

	if (false == is_bt_failed) {
		LOGD("[Recorder] Bluetooth is available");
	}

#if 1
	g_farfieldvoice_h = farfield_voice_init();
	if (NULL == g_farfieldvoice_h) {
		LOGE("[Recorder ERROR] Fail to init farfield_voice_init");
	}

	if (g_farfieldvoice_h) {
		LOGI("[Recorder INFO] Register farfield voice audio callback");
		farfield_voice_register_audio_cb(g_farfieldvoice_h, _ffv_audio_function_cb, NULL);
	}
#endif

	set_audio_source_type(VC_AUDIO_ID_BLUETOOTH);
}

void dependency_tv_audio_deinitialize()
{
	int ret;
	if (g_virtual_sound_stream) {
		sound_manager_stop_virtual_stream(g_virtual_sound_stream);
		ret = sound_manager_destroy_virtual_stream(g_virtual_sound_stream);
		if (0 != ret) {
			LOGE("[Audio ERROR] Fail to destroy virtual stream, ret(%d)", ret);
		}
		g_virtual_sound_stream = NULL;
	}

	if (g_volume_stream) {
		ret = sound_manager_destroy_stream_information(g_volume_stream);
		if (0 != ret) {
			LOGE("[Audio ERROR] Fail to destroy stream information, ret(%d)", ret);
		}
		g_volume_stream = NULL;
	}

#if 1
	if (NULL != g_farfieldvoice_h) {
		LOGD("[Recorder INFO] Unregister farfield voice");
		farfield_voice_unregister_audio_cb(g_farfieldvoice_h);
		farfield_voice_final(g_farfieldvoice_h);
		g_farfieldvoice_h = NULL;
	}
#endif

	bt_hid_unset_audio_data_receive_cb();
	bt_hid_host_deinitialize();
	bt_product_deinit();
}

static void recorder_thread_func()
{
	const int FRAME_LENGTH = 160;
	const int BUFFER_LENGTH = FRAME_LENGTH * 2;
}

void dependency_tv_audio_start_recording()
{
	/* Do not start normal recorder thread if TV_PRODUCT and g_voice_key_pressed,
		just send bt_hid start message */
	if (g_voice_key_pressed) {
		const unsigned char input_data[2] = {SMART_CONTROL_START_CMD, 0x00};
		int bt_retry = 0;
		const int max_retry = 5;
		while (max_retry > bt_retry) {
			int ret = bt_hid_send_rc_command(NULL, input_data, sizeof(input_data));
			if (BT_ERROR_NONE == ret) {
				LOGD("[Recorder] Start bt audio recorder");
				break;
			} else if (BT_ERROR_NOW_IN_PROGRESS == ret) {
				LOGE("[Recorder ERROR] Fail bt_hid_send_rc_command : %d", ret);
				this_thread::sleep_for(chrono::milliseconds(50));
				bt_retry++;
			} else {
				break;
			}
		}
		if (max_retry == bt_retry) {
			LOGE("[Recorder ERROR] Fail to start bt audio");
			return;
		}

		g_bt_extend_count = 0;
	}
#ifdef BUF_SAVE_MODE
	if (g_pFile) {
		fclose(g_pFile);
		g_pFile = NULL;
	} else {
		LOGD("[Recorder Info] File not found!");
	}

	while (1) {
		snprintf(g_temp_file_name, sizeof(g_temp_file_name), "/tmp/dependency_tv_%s_%d_%d", g_audio_source_type, getpid(), g_count);
		int ret = access(g_temp_file_name, 0);

		if (0 == ret) {
			LOGD("[Recorder ERROR] File is already exist");
			if (0 == remove(g_temp_file_name)) {
				LOGD("[Recorder] Remove file");
				break;
			} else {
				g_count++;
			}
		} else {
			break;
		}
	}

	LOGD("[Recorder] Temp file name=[%s]", g_temp_file_name);

	/* open test file */
	g_pFile = fopen(g_temp_file_name, "wb+x");
	if (!g_pFile) {
		LOGD("[Recorder ERROR] File not found!");
		return;
	}
	g_count++;
#endif
}

void dependency_tv_audio_stop_recording()
{
}

void dependency_tv_audio_set_recording_session(unsigned int session)
{
	LOGD("Current recording session : [%s]",
		(RECORDING_SESSION_WAKE_WORD == session) ? "wake word detecting" :
		(RECORDING_SESSION_UTTERANCE == session) ? "utterance recording" :
		(RECORDING_SESSION_FOLLOW_UP == session) ? "follow-up speech recording" :
		"ERROR - Invalid recording session information"
	);
}

void dependency_tv_audio_set_background_volume(double ratio)
{
	int ret;

	if (ratio >= 0.5) {
		if (g_virtual_sound_stream) {
			ret = sound_manager_stop_virtual_stream(g_virtual_sound_stream);
			if (0 != ret) {
					LOGD("[Audio ERROR] Fail to stop virtual stream, ret(%d)", ret);
			}
			ret = sound_manager_destroy_virtual_stream(g_virtual_sound_stream);
			if (0 != ret) {
					LOGD("[Audio ERROR] Fail to destroy virtual stream, ret(%d)", ret);
					return;
			}
			g_virtual_sound_stream = NULL;
		}

		if (g_volume_stream) {
			ret = sound_manager_destroy_stream_information(g_volume_stream);
			if (0 != ret) {
					LOGD("[Audio ERROR] Fail to destroy stream information, ret(%d)", ret);
					return;
			}
			g_volume_stream = NULL;
		}
	} else {
		if (!g_volume_stream) {
			ret = sound_manager_create_stream_information_internal(SOUND_STREAM_TYPE_VOICE_RECOGNITION_SERVICE, NULL, NULL, &g_volume_stream);
			if (0 != ret) {
				LOGD("[Audio] Fail to create stream information, ret(%d)", ret);
				return;
			}
		}

		if (!g_virtual_sound_stream) {
			ret = sound_manager_create_virtual_stream(g_volume_stream, &g_virtual_sound_stream);
			if (0 != ret) {
				LOGD("[Audio ERROR] Fail to create virtual stream, ret(%d)", ret);
				return;
			}
		}
		if (g_virtual_sound_stream) {
			ret = sound_manager_start_virtual_stream(g_virtual_sound_stream);
			if (0 != ret) {
				LOGD("[Audio ERROR] Fail to start virtual stream, ret(%d)", ret);
				return;
			}
		}
	}
}

void dependency_tv_audio_voice_key_pressed_set(bool pressed)
{
	LOGE("ENTER : %d", pressed);
	if (true == g_voice_key_pressed && false == pressed) {
		bt_hid_rc_stop_sending_voice(NULL);
	}

	if (pressed != g_voice_key_pressed) {
		if (pressed) {
			set_audio_source_type(VC_AUDIO_ID_BLUETOOTH);
			dependency_tv_audio_set_background_volume(0.3);
		} else {
			dependency_tv_audio_set_background_volume(1.0);
		}
	}

	g_voice_key_pressed = pressed;
	LOGE("EXIT : %d", g_voice_key_pressed);
}

void dependency_tv_audio_get_audio_format(int* rate, int* channel, int* audio_type)
{
	if (!audio_type || !rate || !channel) {
		return;
	}
	*rate = 16000;
	*channel = 0;
	*audio_type = 0;
}

void dependency_tv_audio_get_audio_source_type(char** type)
{
	if (!type) {
		return;
	}
	*type = g_audio_source_type;
}

