#include "dependency_tv.h"
#include "dependency_tv_button.h"
#include "dependency_tv_audio.h"

#include <chrono>

#include <dlog.h>

static mas_proxy_interface g_proxy_interface;

#define EFL_BETA_API_SUPPORT

#include <Ecore_Input.h>
#include <Ecore_Wl2.h>
#include <Key_Mode.h>

static Ecore_Event_Handler* _key_down_handler = NULL;
static Ecore_Event_Handler* _key_up_handler = NULL;

static long get_current_milliseconds_after_epoch()
{
	auto now = std::chrono::system_clock::now();
	auto now_ms = std::chrono::time_point_cast<std::chrono::milliseconds>(now);
	/* number of milliseconds since the epoch of system_clock */
	auto value = now_ms.time_since_epoch();

	return value.count();
}

static Eina_Bool _key_down_cb(void* data, int type, void* event)
{
	Ecore_Event_Key *ev = (Ecore_Event_Key *) event;
	if (ev) {
		LOGE("KEY[%s], typep[%d]", ev->keyname, type);

		if (ev->keyname && strncmp(ev->keyname, KEY_BT_VOICE, strlen(KEY_BT_VOICE)) == 0 ) {
			dependency_tv_audio_voice_key_pressed_set(true);
			if (g_proxy_interface.process_event) {
				g_proxy_interface.process_event(MA_PLUGIN_EVENT_VOICE_KEY_PRESSED, NULL, 0);
			}
		}
	}

	return ECORE_CALLBACK_DONE;
}

static Eina_Bool _key_up_cb(void* data, int type, void* event)
{
	Ecore_Event_Key *ev = (Ecore_Event_Key *) event;
	if (ev) {
		LOGE("KEY[%s], typep[%d]", ev->keyname, type);

		if (ev->keyname && strncmp(ev->keyname, KEY_BT_VOICE, strlen(KEY_BT_VOICE)) == 0) {
			dependency_tv_audio_voice_key_pressed_set(false);
			long key_up_time = get_current_milliseconds_after_epoch();
			ma_plugin_event_e event = MA_PLUGIN_EVENT_VOICE_KEY_RELEASED_AFTER_PUSH;
			if (g_proxy_interface.process_event) {
				g_proxy_interface.process_event(event, NULL, 0);
			}
		}
	}
	return ECORE_CALLBACK_DONE;
}

static bool _grab_voice_key(void)
{
	Eina_Bool bRet = true;
	bRet = ecore_wl2_window_keygrab_set(NULL,  KEY_BT_VOICE,  0, 0,  0, ECORE_WL2_WINDOW_KEYGRAB_SHARED);
	LOGD("ecore_wl2_window_keygrab_set ret[%d] [%s]", bRet, KEY_BT_VOICE);
	return bRet;
}

static bool _ungrab_voice_key(void)
{
	Eina_Bool bRet = true;
	bRet = ecore_wl2_window_keygrab_unset(NULL,  KEY_BT_VOICE,	0, 0);
	LOGD("ecore_wl2_window_keygrab_unset ret[%d] [%s]", bRet, KEY_BT_VOICE);
	return bRet;
}

static bool _add_key_cb()
{
	if (_key_down_handler == NULL)
	{
		LOGE("_key_down_handler");
		_key_down_handler = ecore_event_handler_add(ECORE_EVENT_KEY_DOWN, _key_down_cb, NULL);
		if(_key_down_handler == NULL)
		{
			LOGE("_key_down_handler == NULL ");
		}
	}

	if (_key_up_handler == NULL)
	{
		LOGE("_key_down_handler");
		_key_up_handler = ecore_event_handler_add(ECORE_EVENT_KEY_UP, _key_up_cb, NULL);
		if(_key_up_handler == NULL)
		{
			LOGE("_key_up_handler == NULL ");
		}
	}
	return true;
}

static bool _delete_key_cb(void)
{
	LOGE("start");
	if (_key_down_handler != NULL)
	{
		ecore_event_handler_del(_key_down_handler);
		_key_down_handler = NULL;
	}

	if (_key_up_handler != NULL)
	{
		ecore_event_handler_del(_key_up_handler);
		_key_up_handler = NULL;
	}
	LOGE("end");
	return true;
}

void dependency_tv_button_initialize(mas_proxy_interface interface)
{
	g_proxy_interface = interface;
	Ecore_Wl2_Display *_ecore_wl2_display = NULL;

	Eina_Bool bRet = ecore_wl2_init();
	LOGD("ecore_wl2_init: %d", bRet);

	_ecore_wl2_display = ecore_wl2_display_connect(NULL);
	LOGD("_ecore_wl2_display: %p", _ecore_wl2_display);

	_grab_voice_key();
	_add_key_cb();
}

void dependency_tv_button_deinitialize()
{
	_delete_key_cb();
	_ungrab_voice_key();
}
