/*
 * Copyright 2018  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef _DEPENDENCY_RESOLVER_H_
#define _DEPENDENCY_RESOLVER_H_

#include <tzplatform_config.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef void (*mas_dependency_error_cb)(int error, const char* err_msg, void* user_data);

typedef int (*mas_proxy_process_event)(int event, void* data, int len);
typedef int (*mas_proxy_feed_audio_data)(wakeup_speech_streaming_event_e event, void* buffer, int len);

typedef struct {
	mas_proxy_process_event						process_event;
	mas_proxy_feed_audio_data					feed_audio_data;
} mas_proxy_interface;

int dependency_resolver_initialize(mas_proxy_interface interface);
int dependency_resolver_deinitialize(void);
int dependency_resolver_set_error_callback(mas_dependency_error_cb callback, void* user_data);
int dependency_resolver_start_recording();
int dependency_resolver_stop_recording();
int dependency_resolver_set_recording_session(unsigned int);
int dependency_resolver_set_background_volume(double ratio);
int dependency_resolver_get_audio_format(int* rate, int* channel, int* audio_type);
int dependency_resolver_get_audio_source_type(char** type);

/**************************************************************************************
 *** Definitions for dependencies
 *************************************************************************************/
#define MAS_DEPENDENCY_DEFAULT_PATH tzplatform_mkpath(tzplatform_getid("TZ_SYS_RO_SHARE"), "multiassistant/")
#ifdef TV_PRODUCT
#define MAS_DEPENDENCY_DEFAULT_FILENAME "libma-dependency-tv.so"
#else
#define MAS_DEPENDENCY_DEFAULT_FILENAME "libma-dependency-default.so"
#endif
#define MAS_DEPENDENCY_MODULE_PATH "db/multi-assistant/dependency_module_path"

#define MAS_DEPENDENCY_FUNC_INITIALIZE "mas_dependency_initialize"
typedef int (*mas_dependency_initialize)(mas_proxy_interface interface, int *dependency_version);
#define MAS_DEPENDENCY_FUNC_DEINITIALIZE "mas_dependency_deinitialize"
typedef int (*mas_dependency_deinitialize)(void);
#define MAS_DEPENDENCY_FUNC_SET_ERROR_CALLBACK "mas_dependency_set_error_callback"
typedef int (*mas_dependency_set_error_callback)(mas_dependency_error_cb callback, void* user_data);
#define MAS_DEPENDENCY_FUNC_START_RECORDING "mas_dependency_start_recording"
typedef int (*mas_dependency_start_recording)(void);
#define MAS_DEPENDENCY_FUNC_STOP_RECORDING "mas_dependency_stop_recording"
typedef int (*mas_dependency_stop_recording)(void);
#define MAS_DEPENDENCY_FUNC_SET_RECORDING_SESSION "mas_dependency_set_recording_session"
typedef int (*mas_dependency_set_recording_session)(unsigned int);
#define MAS_DEPENDENCY_FUNC_SET_BACKGROUND_VOLUME "mas_dependency_set_background_volume"
typedef int (*mas_dependency_set_background_volume)(double ratio);
#define MAS_DEPENDENCY_FUNC_GET_AUDIO_FORMAT "mas_dependency_get_audio_format"
typedef int (*mas_dependency_get_audio_format)(int* rate, int* channel, int* audio_type);
#define MAS_DEPENDENCY_FUNC_GET_AUDIO_SOURCE_TYPE "mas_dependency_get_audio_source_type"
typedef int (*mas_dependency_get_audio_source_type)(char** type);

typedef struct {
	mas_dependency_initialize								initialize;
	mas_dependency_deinitialize								deinitialize;
	mas_dependency_set_error_callback						set_error_callback;
	mas_dependency_start_recording							start_recording;
	mas_dependency_stop_recording							stop_recording;
	mas_dependency_set_recording_session					set_recording_session;
	mas_dependency_set_background_volume					set_background_volume;
	mas_dependency_get_audio_format							get_audio_format;
	mas_dependency_get_audio_source_type					get_audio_source_type;
} mas_dependency;

#ifdef __cplusplus
}
#endif

#endif /* _DEPENDENCY_RESOLVER_H_ */
