/*
 * Copyright 2018  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _WAKEUP_POLICY_H_
#define _WAKEUP_POLICY_H_

#include <string>
#include <memory>

#include "wakeup_interfaces.h"

namespace multiassistant
{
namespace wakeup
{

using namespace std;

class IPolicyEventObserver
{
public:
	virtual ~IPolicyEventObserver() = default;
	virtual void on_wakeup(wakeup_event_info wakeup_info) = 0;
};

class CWakeupPolicyImpl;

class CWakeupPolicy
{
public:
	CWakeupPolicy();
	CWakeupPolicy(IPolicyEventObserver *observer);
	virtual ~CWakeupPolicy();

	CWakeupPolicy(const CWakeupPolicy&) = delete;
	CWakeupPolicy& operator=(const CWakeupPolicy&) = delete;

	void subscribe(IPolicyEventObserver *observer);
	void unsubscribe(IPolicyEventObserver *observer);

	virtual void wakeup_candidate(wakeup_event_info wakeup_info) = 0;
protected:
	unique_ptr<CWakeupPolicyImpl> mImpl;
};

} // wakeup
} // multiassistant

#endif /* _WAKEUP_POLICY_H_ */
