/*
 * Copyright 2018  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#ifndef _WAKEUP_SETTINGS_H_
#define _WAKEUP_SETTINGS_H_

#include <string>
#include <vector>
#include <vconf.h>

typedef void (*input_language_changed_cb)(void* data);

namespace multiassistant
{
namespace wakeup
{

using namespace std;

#define WAKEUP_SETTINGS_KEY_DEFAULT_ASSISTANT_APPID "db/multi-assistant/default_assistant_appid"
#define WAKEUP_SETTINGS_KEY_UI_PANEL_ENABLED "db/multi-assistant/ui_panel_enabled"
#define WAKEUP_SETTINGS_KEY_CONVERSATION_TIMEOUT "db/multi-assistant/conversation_timeout"
#define WAKEUP_SETTINGS_KEY_MULTIPLE_MODE "db/multi-assistant/multiple_mode"
#define WAKEUP_SETTINGS_KEY_ENABLED_ASSISTANTS "db/multi-assistant/enabled_assistants"
#define WAKEUP_SETTINGS_KEY_WAKEUP_POLICY_DELAY "db/multi-assistant/wakeup_policy_delay"
#define WAKEUP_SETTINGS_KEY_WAKEUP_POLICY_PRIORITY "db/multi-assistant/wakeup_policy_priority"
#define WAKEUP_SETTINGS_KEY_STREAMING_DURATION_MAX "db/multi-assistant/streaming_duration_max"
#define WAKEUP_SETTINGS_KEY_VOICE_INPUT_LANGUAGE "db/multi-assistant/voice_input_language"

class ISettingsEventObserver
{
public:
	virtual ~ISettingsEventObserver() = default;
	virtual bool on_voice_input_language_changed(const char* language) = 0;
	virtual bool on_assistant_enabled_info_changed(const char* appid, bool enabled) = 0;
	virtual bool on_default_assistant_appid_changed(const char* appid) = 0;
};

class CWakeupSettings
{
public:
	CWakeupSettings();
	virtual ~CWakeupSettings();

	CWakeupSettings(const CWakeupSettings&) = delete;
	CWakeupSettings& operator=(const CWakeupSettings&) = delete;

	void initialize();
	void deinitialize();

	void subscribe(ISettingsEventObserver *observer);
	void unsubscribe(ISettingsEventObserver *observer);
	vector<ISettingsEventObserver*> get_observers();

	string get_default_assistant_appid();
	bool get_ui_panel_enabled();
	float get_conversation_timeout();
	bool get_multiple_mode();
	vector<string> get_enabled_assistants();
	float get_wakeup_policy_delay();
	vector<string> get_wakeup_policy_priority();
	float get_streaming_duration_max();
	string get_current_language(void);
private:
	vector<ISettingsEventObserver*> mObservers;

	string mDefaultAssistantAppid;
	bool mUiPanelEnabled{true};
	float mConversationTimeout{5.0};
	bool mMultipleMode{false};
	vector<string> mEnabledAssistants;
	float mWakeupPolicyDelay{0.1};
	vector<string> mWakeupPolicyPriority; // No priority by default
	float mStreamingDurationMax{10.0};
	string mVoiceInputLanguage;
};

} // wakeup
} // multiassistant

#endif /* _WAKEUP_SETTINGS_H_ */
