#include "wakeup_policy.h"
#include "wakeup_policy_impl.h"

#include <vector>
#include <algorithm>

namespace multiassistant
{
namespace wakeup
{

void CWakeupPolicyImpl::subscribe(IPolicyEventObserver *observer)
{
	mObservers.push_back(observer);
}

void CWakeupPolicyImpl::unsubscribe(IPolicyEventObserver *observer)
{
	auto iter = find(mObservers.begin(), mObservers.end(), observer);
	if (iter != mObservers.end()) {
		mObservers.erase(iter);
	}
}

void CWakeupPolicyImpl::wakeup(wakeup_event_info wakeup_info)
{
	for (const auto& observer : mObservers) {
		if (observer) {
			observer->on_wakeup(wakeup_info);
		}
	}
}

CWakeupPolicy::CWakeupPolicy()
{
	mImpl.reset(new CWakeupPolicyImpl);
}

CWakeupPolicy::~CWakeupPolicy()
{
}

CWakeupPolicy::CWakeupPolicy(IPolicyEventObserver *observer) : CWakeupPolicy()
{
	subscribe(observer);
}

void CWakeupPolicy::subscribe(IPolicyEventObserver *observer)
{
	if (mImpl) mImpl->subscribe(observer);
}

void CWakeupPolicy::unsubscribe(IPolicyEventObserver *observer)
{
	if (mImpl) mImpl->unsubscribe(observer);
}

} // wakeup
} // multiassistant
