#include "wakeup_policy_default.h"
#include "wakeup_policy_impl.h"

#include <vector>

#include <Eina.h>
#include <dlog.h>

#ifndef LOG_TAG
#define LOG_TAG "WakeupPolicyDefault"
#endif

namespace multiassistant
{
namespace wakeup
{

CWakeupPolicyDefault::CWakeupPolicyDefault()
{
}

CWakeupPolicyDefault::CWakeupPolicyDefault(IPolicyEventObserver *observer)
	: CWakeupPolicy(observer)
{
}

CWakeupPolicyDefault::~CWakeupPolicyDefault()
{
	if (mTimer) {
		ecore_timer_del(mTimer);
		mTimer = nullptr;
	}
}

void CWakeupPolicyDefault::set_assistant_priority(string appid, int priority)
{
	PRIORITY_INFO info;
	info.appid = appid;
	info.priority = priority;

	LOGD("Setting assistant priority : %s %d", appid.c_str(), priority);
	mPriorityInfos.push_back(info);
}

void CWakeupPolicyDefault::set_delay(float seconds)
{
	LOGD("Setting delay : %f", seconds);
	mDelaySeconds = seconds;
}

static Eina_Bool timer_func(void *data)
{
	LOGD("[ENTER]");
	if (data) {
		CWakeupPolicyDefault *policy = static_cast<CWakeupPolicyDefault*>(data);
		policy->timer_expired();
	}
	return ECORE_CALLBACK_CANCEL;
}

void CWakeupPolicyDefault::wakeup_candidate(wakeup_event_info wakeup_info)
{
	mWakeupInfos.push_back(wakeup_info);
	if (nullptr == mTimer) {
		ecore_thread_main_loop_begin();
		mTimer = ecore_timer_add(mDelaySeconds, timer_func, this);
		ecore_thread_main_loop_end();
	}
}

void CWakeupPolicyDefault::timer_expired()
{
	LOGD("[ENTER]");
	if (0 == mWakeupInfos.size()) return;

	wakeup_event_info selected = mWakeupInfos.front();
	int selected_priority = -1;
	for (const auto &wakeup : mWakeupInfos) {
		for (const auto &info : mPriorityInfos) {
			if (0 == info.appid.compare(wakeup.wakeup_appid)) {
				if (info.priority > selected_priority || -1 == selected_priority) {
					LOGD("Wakeup with higher priority found : %s, %d",
						info.appid.c_str(), info.priority);
					selected = wakeup;
					selected_priority = info.priority;
				}
			}
		}
	}
	if (mImpl) mImpl->wakeup(selected);
	mWakeupInfos.clear();

	if (mTimer) {
		LOGD("Now deleting timer");
		ecore_timer_del(mTimer);
		mTimer = nullptr;
	}
}

} // wakeup
} // multiassistant
