/*
 * Copyright 2018  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.1 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://floralicense.org/license/
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include "multi_assistant_main.h"
#include "multi_assistant_dbus.h"
#include "multi_assistant_dbus_server.h"
#include "multi_assistant_service.h"

/*
* Dbus Client-Daemon Server
*/
int ma_service_dbus_hello(DBusConnection* conn, DBusMessage* msg)
{
	MAS_LOGD("[DEBUG] MAS HELLO");

	DBusMessage* reply;
	reply = dbus_message_new_method_return(msg);

	if (NULL != reply) {
		if (!dbus_connection_send(conn, reply, NULL)) {
			MAS_LOGE("[OUT ERROR] Out Of Memory!");
		}

		dbus_connection_flush(conn);
		dbus_message_unref(reply);
	} else {
		MAS_LOGE("[OUT ERROR] Fail to create reply message!!");
	}

	return 0;
}

int ma_service_dbus_initialize(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid = -1;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS INITIALIZE");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas initialize : get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas initialize : pid(%d)", pid);
		ret =  mas_client_initialize(pid);
	}

	DBusMessage* reply;
	reply = dbus_message_new_method_return(msg);

 	MAS_LOGD("[OUT] mas initialize : result(%d)", ret);
	if (NULL != reply) {
		dbus_message_append_args(reply,
			DBUS_TYPE_INT32, &ret,
			DBUS_TYPE_INVALID);

		if (0 == ret) {
			MAS_LOGD("[OUT] mas initialize : result(%d)", ret);
		} else {
			MAS_LOGE("[OUT ERROR] mas initialize : result(%d)", ret);
		}

		if (!dbus_connection_send(conn, reply, NULL)) {
			MAS_LOGE("[OUT ERROR] mas initialize : Out Of Memory!");
		}

		dbus_connection_flush(conn);
		dbus_message_unref(reply);
	} else {
		MAS_LOGE("[OUT ERROR] mas initialize : Fail to create reply message!!");
	}

	return 0;
}

int ma_service_dbus_deinitialize(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid = -1;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS DEINITIALIZE");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas deinitialize : get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas deinitialize : pid(%d)", pid);
		ret =  mas_client_deinitialize(pid);
	}

	DBusMessage* reply;
	reply = dbus_message_new_method_return(msg);

	MAS_LOGD("[OUT] mas deinitialize : result(%d)", ret);
	if (NULL != reply) {
		dbus_message_append_args(reply,
			DBUS_TYPE_INT32, &ret,
			DBUS_TYPE_INVALID);

		if (0 == ret) {
			MAS_LOGD("[OUT] mas deinitialize : result(%d)", ret);
		} else {
			MAS_LOGE("[OUT ERROR] mas deinitialize : result(%d)", ret);
		}

		if (!dbus_connection_send(conn, reply, NULL)) {
			MAS_LOGE("[OUT ERROR] mas deinitialize : Out Of Memory!");
		}

		dbus_connection_flush(conn);
		dbus_message_unref(reply);
	} else {
		MAS_LOGE("[OUT ERROR] mas deinitialize : Fail to create reply message!!");
	}

	return 0;
}

int ma_service_dbus_get_audio_format(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid = -1;
	int rate, channel, audio_type;
	int ret;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS GET AUDIO FORMAT");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas get audio format : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas get default voice");
		ret = mas_client_get_audio_format(pid, &rate, &channel, &audio_type);
	}

	DBusMessage* reply;
	reply = dbus_message_new_method_return(msg);

	if (NULL != reply) {
		/* Append result and voice */
		dbus_message_append_args(reply,
			DBUS_TYPE_INT32, &ret,
			DBUS_TYPE_INT32, &rate,
			DBUS_TYPE_INT32, &channel,
			DBUS_TYPE_INT32, &audio_type,
			DBUS_TYPE_INVALID);
		if (0 == ret) {
			MAS_LOGD("[OUT] mas get audio format : rate(%d), channel(%d), audio_type(%d)", rate, channel, audio_type);
		} else {
			MAS_LOGE("[OUT ERROR] mas get audio format : result(%d)", ret);
		}

		if (!dbus_connection_send(conn, reply, NULL)) {
			MAS_LOGE("[OUT ERROR] mas get audio format : Out Of Memory!");
		}

		dbus_connection_flush(conn);
		dbus_message_unref(reply);
	} else {
		MAS_LOGE("[OUT ERROR] tts default voice : Fail to create reply message!!");
	}

	return 0;
}

int ma_service_dbus_get_audio_source_type(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid = -1;
	char *type = NULL;
	int ret;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS GET AUDIO SOURCE TYPE");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas get audio source type : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas get default voice");
		ret = mas_client_get_audio_source_type(pid, &type);
	}

	DBusMessage* reply;
	reply = dbus_message_new_method_return(msg);

	char* temp_type = NULL;

	if (type)
		temp_type = strdup(type);
	else
		temp_type = strdup("#NULL");

	if (NULL != reply) {
		/* Append result and voice */
		dbus_message_append_args(reply,
			DBUS_TYPE_INT32, &ret,
			DBUS_TYPE_STRING, &temp_type,
			DBUS_TYPE_INVALID);
		if (0 == ret) {
			MAS_LOGD("[OUT] mas get audio source type : type(%s)", temp_type);
		} else {
			MAS_LOGE("[OUT ERROR] mas get audio source type : result(%d)", ret);
		}

		if (!dbus_connection_send(conn, reply, NULL)) {
			MAS_LOGE("[OUT ERROR] mas get audio source type : Out Of Memory!");
		}

		dbus_connection_flush(conn);
		dbus_message_unref(reply);
	} else {
		MAS_LOGE("[OUT ERROR] mas get audio source type : Fail to create reply message!!");
	}

	if (temp_type)
		free(temp_type);

	return 0;
}

int ma_service_dbus_send_asr_result(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid, event;
	char *asr_result;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INT32, &event,
		DBUS_TYPE_STRING, &asr_result,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SEND ASR RESULT");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas send asr result : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas send asr result : pid(%d), event(%d), asr_result(%s)", pid, event, asr_result);
		ret = mas_client_send_asr_result(pid, event, asr_result);
	}

	DBusMessage* reply;
	reply = dbus_message_new_method_return(msg);

	if (NULL != reply) {
		dbus_message_append_args(reply,
			DBUS_TYPE_INT32, &ret,
			DBUS_TYPE_INVALID);

		if (0 == ret) {
			MAS_LOGD("[OUT] mas send asr result : result(%d)", ret);
		} else {
			MAS_LOGE("[OUT ERROR] mas send asr result : result(%d)", ret);
		}

		if (!dbus_connection_send(conn, reply, NULL)) {
			MAS_LOGE("[OUT ERROR] mas send asr result : Out Of Memory!");
		}

		dbus_connection_flush(conn);
		dbus_message_unref(reply);
	} else {
		MAS_LOGE("[OUT ERROR] mas send asr result : Fail to create reply message!!");
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_send_result(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	char* display_text;
	char* utterance_text;
	char* result_json;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_STRING, &display_text,
		DBUS_TYPE_STRING, &utterance_text,
		DBUS_TYPE_STRING, &result_json,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SEND RESULT");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas send result : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas send result : pid(%d), display_text(%s), utterance_text(%s), result_json(%s)", pid, display_text, utterance_text, result_json);
		ret =  mas_client_send_result(pid, display_text, utterance_text, result_json);
	}

	DBusMessage* reply;
	reply = dbus_message_new_method_return(msg);

	if (NULL != reply) {
		dbus_message_append_args(reply,
			DBUS_TYPE_INT32, &ret,
			DBUS_TYPE_INVALID);

		if (0 == ret) {
			MAS_LOGD("[OUT] mas send result : result(%d)", ret);
		} else {
			MAS_LOGE("[OUT ERROR] mas send result : result(%d)", ret);
		}

		if (!dbus_connection_send(conn, reply, NULL)) {
			MAS_LOGE("[OUT ERROR] mas send result : Out Of Memory!");
		}

		dbus_connection_flush(conn);
		dbus_message_unref(reply);
	} else {
		MAS_LOGE("[OUT ERROR] mas send result : Fail to create reply message!!");
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_send_recognition_result(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	int result;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INT32, &result,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SEND RECOGNITION RESULT");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas send error : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas send error : pid(%d), result(%d)", pid, result);
		ret =  mas_client_send_recognition_result(pid, result);
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_start_streaming_audio_data(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	int type;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INT32, &type,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SEND START STREAMING");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas send start streaming : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas send start streaming : pid(%d), type(%d)", pid, type);
		ret =  mas_client_start_streaming_audio_data(pid, type);
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_stop_streaming_audio_data(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	int type;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INT32, &type,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SEND STOP STREAMING");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas stop streaming : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas stop streaming : pid(%d), type(%d)", pid, type);
		ret =  mas_client_stop_streaming_audio_data(pid, type);
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_update_voice_feedback_state(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	int state;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INT32, &state,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SEND UPDATE VOICE FEEDBACK");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas update voice feedback : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas update voice feedback : pid(%d), state(%d)", pid, state);
		ret =  mas_client_update_voice_feedback_state(pid, state);
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_send_assistant_specific_command(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	int ret = 0;
	char* command;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_STRING, &command,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SEND ASSISTANT SPECIFIC COMMAND");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas send assistant specific command : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas send assistant specific command : pid(%d), command(%s)", pid, command);
		ret =  mas_client_send_assistant_specific_command(pid, command);
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_set_background_volume(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	int ret = 0;
	double ratio;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_DOUBLE, &ratio,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SET BACKGROUND VOLUME");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas set background volume : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas set background volume : pid(%d), ratio(%f)", pid, ratio);
		ret =  mas_client_set_background_volume(pid, ratio);
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_set_preprocessing_allow_mode(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	int ret = 0;
	int mode;
	char* app_id;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INT32, &mode,
		DBUS_TYPE_STRING, &app_id,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SET PREPROCESSING ALLOW MODE");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas set preprocessing allow mode : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		char* temp_app_id = NULL;
		if (NULL != app_id && strcmp("#NULL", app_id)) {
			temp_app_id = strdup(app_id);
		}
		MAS_LOGD("[IN] mas set preprocessing allow mode : pid(%d), mode(%d), app_id(%s)", pid, mode, temp_app_id);
		ret =  mas_client_set_preprocessing_allow_mode(pid, mode, temp_app_id);
		if (NULL != temp_app_id)
			free(temp_app_id);
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_send_preprocessing_result(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	int ret = 0;
	int result;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INT32, &result,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SEND PREPROCESSING RESULT");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas send preprocessing result : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas send preprocessing result : pid(%d), result(%d)", pid, result);
		ret =  mas_client_send_preprocessing_result(pid, (bool)result);
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_set_wake_word_audio_require_flag(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	int ret = 0;
	int require;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INT32, &require,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SET WAKE WORD AUDIO REQUIRE FLAG");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas set wake word audio require flag : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas set wake word audio require flag : pid(%d), require(%d)", pid, require);
		ret =  mas_client_set_wake_word_audio_require_flag(pid, (bool)require);
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_dbus_set_assistant_language(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid;
	int ret = 0;
	char* language;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_STRING, &language,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS SET ASSISTANT LANGUAGE");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas set assistant language : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas set assistant language : pid(%d), language(%s)", pid, language);
		ret =  mas_client_set_assistant_language(pid, language);
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}

int ma_service_ui_dbus_initialize(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid = -1;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS UI INITIALIZE");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas ui initialize : get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas ui initialize : pid(%d)", pid);
		ret =  mas_ui_client_initialize(pid);
	}

	DBusMessage* reply;
	reply = dbus_message_new_method_return(msg);

	MAS_LOGD("[OUT] mas ui initialize : result(%d)", ret);
	if (NULL != reply) {
		dbus_message_append_args(reply,
			DBUS_TYPE_INT32, &ret,
			DBUS_TYPE_INVALID);

		if (0 == ret) {
			MAS_LOGD("[OUT] mas ui initialize : result(%d)", ret);
		} else {
			MAS_LOGE("[OUT ERROR] mas ui initialize : result(%d)", ret);
		}

		if (!dbus_connection_send(conn, reply, NULL)) {
			MAS_LOGE("[OUT ERROR] mas ui initialize : Out Of Memory!");
		}

		dbus_connection_flush(conn);
		dbus_message_unref(reply);
	} else {
		MAS_LOGE("[OUT ERROR] mas ui initialize : Fail to create reply message!!");
	}

	return 0;
}

int ma_service_ui_dbus_deinitialize(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	int pid = -1;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_INT32, &pid,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS UI DEINITIALIZE");

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas ui deinitialize : get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas ui deinitialize : pid(%d)", pid);
		ret =  mas_ui_client_deinitialize(pid);
	}

	DBusMessage* reply;
	reply = dbus_message_new_method_return(msg);

	MAS_LOGD("[OUT] mas ui deinitialize : result(%d)", ret);
	if (NULL != reply) {
		dbus_message_append_args(reply,
			DBUS_TYPE_INT32, &ret,
			DBUS_TYPE_INVALID);

		if (0 == ret) {
			MAS_LOGD("[OUT] mas ui deinitialize : result(%d)", ret);
		} else {
			MAS_LOGE("[OUT ERROR] mas ui deinitialize : result(%d)", ret);
		}

		if (!dbus_connection_send(conn, reply, NULL)) {
			MAS_LOGE("[OUT ERROR] mas ui deinitialize : Out Of Memory!");
		}

		dbus_connection_flush(conn);
		dbus_message_unref(reply);
	} else {
		MAS_LOGE("[OUT ERROR] mas ui deinitialize : Fail to create reply message!!");
	}

	return 0;
}

int ma_service_ui_dbus_change_assistant(DBusConnection* conn, DBusMessage* msg)
{
	DBusError err;
	dbus_error_init(&err);

	char *app_id;
	int ret = 0;

	dbus_message_get_args(msg, &err,
		DBUS_TYPE_STRING, &app_id,
		DBUS_TYPE_INVALID);

	MAS_LOGD("[DEBUG] MAS UI CHANGE ASSISTANT : %s", (app_id ? app_id : "NULL"));

	if (dbus_error_is_set(&err)) {
		MAS_LOGE("[IN ERROR] mas ui change assisant : Get arguments error (%s)", err.message);
		dbus_error_free(&err);
		ret = -1; //MAS_ERROR_OPERATION_FAILED;
	} else {
		MAS_LOGD("[IN] mas ui change assisant : app_id(%s)", app_id);
		ret = mas_ui_client_change_assistant(app_id);
	}

	DBusMessage* reply;
	reply = dbus_message_new_method_return(msg);

	if (NULL != reply) {
		dbus_message_append_args(reply,
			DBUS_TYPE_INT32, &ret,
			DBUS_TYPE_INVALID);

		if (0 == ret) {
			MAS_LOGD("[OUT] mas ui change assisant : result(%d)", ret);
		} else {
			MAS_LOGE("[OUT ERROR] mas ui change assisant : result(%d)", ret);
		}

		if (!dbus_connection_send(conn, reply, NULL)) {
			MAS_LOGE("[OUT ERROR] mas ui change assisant : Out Of Memory!");
		}

		dbus_connection_flush(conn);
		dbus_message_unref(reply);
	} else {
		MAS_LOGE("[OUT ERROR] mas ui change assisant : Fail to create reply message!!");
	}

	MAS_LOGD("<<<<<");
	MAS_LOGD("  ");

	return 0;
}
